import { UIActionContext } from "@/logic/ui-logic";
/**
 * 自定义表格数据加载
 *
 * @export
 * @class GridViewLoadUILogicBase
 */
export default class GridViewLoadUILogicBase {

    /**
     * Creates an instance of  GridViewLoadBase.
     * 
     * @param {*} [opts={}]
     * @memberof GridViewLoadUILogicBase
     */
    constructor(opts: any = {}) { }

    /**
     * 开始节点
     *
     * @protected
     * @type {string}
     * @memberof GridViewLoadUILogicBase
     */
    protected readonly startLogicNode: string = 'Begin';

    /**
     * 逻辑参数
     *
     * @protected
     * @type {any[]}
     * @memberof GridViewLoadUILogicBase
     */
    protected logicParams: any[] = [
        {
            name: '当前视图参数',
            codeName: 'viewParam',
            activeContainerParam: false,
            activeCtrlParam: false,
            activeViewParam: false,
            appGlobalParam: false,
            applicationParam: false,
            ctrlParam: false,
            default: false,
            entityParam: false,
            entityListParam: false,
            entityPageParam: false,
            filterParam: false,
            navContextParam: false,
            simpleParam: false,
            simpleListParam: false,
            routeViewSessionParam: false,
            viewNavDataParam: true,
            viewSessionParam: false,
            lastReturnParam: false
        },
        {
            name: '当前视图对象',
            codeName: 'view',
            activeContainerParam: false,
            activeCtrlParam: false,
            activeViewParam: true,
            appGlobalParam: false,
            applicationParam: false,
            ctrlParam: false,
            default: false,
            entityParam: false,
            entityListParam: false,
            entityPageParam: false,
            filterParam: false,
            navContextParam: false,
            simpleParam: false,
            simpleListParam: false,
            routeViewSessionParam: false,
            viewNavDataParam: false,
            viewSessionParam: false,
            lastReturnParam: false
        },
        {
            name: '传入变量',
            codeName: 'Default',
            activeContainerParam: false,
            activeCtrlParam: false,
            activeViewParam: false,
            appGlobalParam: false,
            applicationParam: false,
            ctrlParam: false,
            default: true,
            entityParam: true,
            entityListParam: false,
            entityPageParam: false,
            filterParam: false,
            navContextParam: false,
            simpleParam: false,
            simpleListParam: false,
            routeViewSessionParam: false,
            viewNavDataParam: false,
            viewSessionParam: false,
            lastReturnParam: false
        },
        {
            name: '当前表格',
            codeName: 'grid',
            activeContainerParam: false,
            activeCtrlParam: false,
            activeViewParam: false,
            appGlobalParam: false,
            applicationParam: false,
            ctrlParam: true,
            default: false,
            entityParam: false,
            entityListParam: false,
            entityPageParam: false,
            filterParam: false,
            navContextParam: false,
            simpleParam: false,
            simpleListParam: false,
            routeViewSessionParam: false,
            viewNavDataParam: false,
            viewSessionParam: false,
            lastReturnParam: false
        },
        {
            name: '当前搜索表单',
            codeName: 'searchForm',
            activeContainerParam: false,
            activeCtrlParam: false,
            activeViewParam: false,
            appGlobalParam: false,
            applicationParam: false,
            ctrlParam: true,
            default: false,
            entityParam: false,
            entityListParam: false,
            entityPageParam: false,
            filterParam: false,
            navContextParam: false,
            simpleParam: false,
            simpleListParam: false,
            routeViewSessionParam: false,
            viewNavDataParam: false,
            viewSessionParam: false,
            lastReturnParam: false
        },
    ];

    /**
     * 执行前
     *
     * @param {*} args
     * @param {*} [context={}]
     * @param {*} [params={}]
     * @param {*} [$event]
     * @param {*} [xData]
     * @param {*} [actioncontext]
     * @param {string} [srfParentDeName]
     * @return {*} 
     * @memberof GridViewLoadUILogicBase
     */
    public beforeExecute(args: any, context: any = {}, params: any = {},
        $event?: any, xData?: any, actioncontext?: any, srfParentDeName?: string) {
        return new UIActionContext(this.logicParams, args, context, params, $event, xData, actioncontext, srfParentDeName)
    }

    /**
     * 执行
     *
     * @param {any[]} args
     * @param {*} [context={}]
     * @param {*} [params={}]
     * @param {*} [$event]
     * @param {*} [xData]
     * @param {*} [actionContext]
     * @param {string} [srfParentDeName]
     * @memberof GridViewLoadUILogicBase
     */
    async execute(args: any[], context:any = {} ,params: any = {}, $event?: any, xData?: any, actioncontext?: any, srfParentDeName?: string) {
        try {
            const actionContext = this.beforeExecute(args, context, params, $event, xData, actioncontext, srfParentDeName);
            await this.execute_begin_node(actionContext);
            return actionContext.getResult();
        } catch (error: any) {
            throw new Error(`${error && error.message ? error.message : '发生未知错误！'}`);
        }
    }

    /**
     * 开始
     *
     * @param {UIActionContext} actionContext 界面逻辑上下文
     */
    protected async execute_begin_node(actionContext: UIActionContext) {
        //  自定义表格数据加载
    actionContext.setResult(actionContext.defaultParam.getReal());
    console.log('已完成执行 开始 节点');
    await this.execute_bindparam1_node(actionContext);
    }
    /**
     * 绑定当前搜索表单
     *
     * @param {UIActionContext} actionContext 界面逻辑上下文
     */
    protected async execute_bindparam1_node(actionContext: UIActionContext) {
        //  自定义表格数据加载
    try {
        //  源数据
        const srcParam = actionContext.getParam('view');
        //  目标数据
        const dstParam = actionContext.getParam('searchForm');
        //  源属性
        const srcFieldName: string = 'searchform';
        if (srcFieldName) {
            dstParam.bind(srcParam.get(srcFieldName));
        } else {
            dstParam.bind(srcParam.getReal());
        }
        actionContext.bindLastReturnParam(null);
    } catch (error: any) {
        throw new Error('逻辑参数当前搜索表单 ${error && error.message ? error.message : '发生未知错误!'}');
    }
    console.log('已完成执行 绑定当前搜索表单 节点');
    await this.execute_viewctrlinvoke1_node(actionContext);
    }
    /**
     * 视图部件调用
     *
     * @param {UIActionContext} actionContext 界面逻辑上下文
     */
    protected async execute_viewctrlinvoke1_node(actionContext: UIActionContext) {
        //  自定义表格数据加载
    const invokeCtrl = 'searchForm';
    const invokeMethod = 'loadDraft';
    const invokeParam = 'viewParam';
    if (!invokeCtrl || !invokeMethod) {
        throw new Error(`界面对象或者调用方法缺失`);
    }
    const invokeUICtrl = actionContext.getParam(invokeCtrl).getReal();
    if (invokeUICtrl[invokeMethod] && invokeUICtrl[invokeMethod] instanceof Function) {
        try {
            const result = await invokeUICtrl[invokeMethod]();
            if (invokeParam) {
                actionContext.getParam(invokeParam).bind(result);
            }
            actionContext.bindLastReturnParam(result);
        } catch (error:any) {
            throw new Error(`${invokeCtrl}界面对象调用${invokeMethod}方法发生异常`);
        }
    } else {
        throw new Error(`${invokeCtrl}界面对象不存在${invokeMethod}方法`);
    }
    console.log('已完成执行 视图部件调用 节点');
    await this.execute_viewctrlinvoke2_node(actionContext);
    }
    /**
     * 结束
     *
     * @param {UIActionContext} actionContext 界面逻辑上下文
     */
    protected async execute_end1_node(actionContext: UIActionContext) {
        //  自定义表格数据加载
    //  TODO 等待补充
    console.log('已完成执行 结束 节点');
    }
    /**
     * 视图部件调用
     *
     * @param {UIActionContext} actionContext 界面逻辑上下文
     */
    protected async execute_viewctrlinvoke2_node(actionContext: UIActionContext) {
        //  自定义表格数据加载
    const invokeCtrl = 'grid';
    const invokeMethod = 'load';
    const invokeParam = 'viewParam';
    if (!invokeCtrl || !invokeMethod) {
        throw new Error(`界面对象或者调用方法缺失`);
    }
    const invokeUICtrl = actionContext.getParam(invokeCtrl).getReal();
    if (invokeUICtrl[invokeMethod] && invokeUICtrl[invokeMethod] instanceof Function) {
        try {
            const result = await invokeUICtrl[invokeMethod]();
            if (invokeParam) {
                actionContext.getParam(invokeParam).bind(result);
            }
            actionContext.bindLastReturnParam(result);
        } catch (error:any) {
            throw new Error(`${invokeCtrl}界面对象调用${invokeMethod}方法发生异常`);
        }
    } else {
        throw new Error(`${invokeCtrl}界面对象不存在${invokeMethod}方法`);
    }
    console.log('已完成执行 视图部件调用 节点');
    await this.execute_end1_node(actionContext);
    }

}